# A mono-spaced font texture that can be used to render strings.
class FontTexture
	# The constructor expects to find a <em>fontname.png</em> and a <em>fontname.font</em>
	# * Each character should be wrapped in its own single pixel black border. (1 between the edge
	#   and 2 between characters)
	# * The <em>fontname.font</em> should follow this pattern by line.
	#   1. Number of rows in the file.
	#   2. Number of columns in the file.
	#   3. Horizontal spacing for the font.
	#   4. Vertical spacing.
	#   5. Write 'TRUE' if there are capital and lower case letters in this font.
	#      Leave blank otherwise.
	#   6. Format string. List the characters in the font from left to right, bottom to top.
	#      Use spaces for empty characters. 
	def initialize(fontname)
		@tex = AS::Texture.open(fontname + '.png')
		
		formatArr = File.open(fontname + '.font').readlines()
		
		@charsPerRow = Integer( formatArr[0] )
		@charsPerCol  = Integer( formatArr[1] )
		
		@hspace = Integer( formatArr[2] )
		@vspace = Integer( formatArr[3] )
		
		@hasUpperCase = (formatArr[4] == 'TRUE')
		
		@cwidth  = Integer(@tex.w/@charsPerRow)
		@cheight = Integer(@tex.h/@charsPerCol)
		
		@hash = {}; i = -1
		formatArr[5].each_byte do|n|
			i += 1
			next if n==(?\ )
			list = AS.create_list do
				drawChar(i)
			end
			@hash[n] = list
		end
		
		space = AS.create_list do
			GL.Translate(@cwidth + @hspace, 0, 0)
		end
		@hash[?\ ] = space
		@hash.default = space
		
		@hash[?\n] = AS.create_list do
			GL.PopMatrix()
			GL.Translate(0, @cheight + @vspace, 0)
			GL.PushMatrix()
		end
	end
	
	# Draw a single character, used to generate the DLs
	def drawChar(n)
		cy, cx = n.divmod(@charsPerRow)
		cy = @charsPerCol - cy - 1
		GL.Begin(GL::QUADS)
			GL.TexCoord(    @cwidth*cx,     @cheight*cy); GL.Vertex(      0, -@cheight)
			GL.TexCoord(@cwidth*(cx+1),     @cheight*cy); GL.Vertex(@cwidth, -@cheight)
			GL.TexCoord(@cwidth*(cx+1), @cheight*(cy+1)); GL.Vertex(@cwidth,         0)
			GL.TexCoord(    @cwidth*cx, @cheight*(cy+1)); GL.Vertex(      0,         0)
		GL.End
		GL.Translate(@cwidth + @hspace, 0, 0)
	end
	
	# Render a string. (_x_,_y_) is the position of the top-left corner, ande _s_ is the
	# scale.Fractional widths can be used for sub-pixel placement, but use with caution.
	def draw(x, y, s, str)
		@tex.bind()
		GL.PushMatrix()
			GL.Translate(x, y, 0)
			GL.Scale(s,s,0)
			GL.PushMatrix()
				str = str.downcase unless @hasUpperCase
				str.each_byte do|n|
					@hash[n].call_list
				end
			GL.PopMatrix()
		GL.PopMatrix()
	end
	
	# Get the position to render a string centered at (_x_,_y_). May return fractional widths.
	# *WARNING* str cannot contain newline characters.
	def draw_centered(x, y, s, str)
		cx = x - str.length*getWidth*s/2.0
		cy = y + getHeight*s/2.0
		draw(cx, cy, s, str)
	end
	
	# Gets the horizontal spacing of the font.
	def getWidth()
		@cwidth + @hspace
	end
	
	# Gets the vertical spacing of the font.
	def getHeight()
		@cheight + @vspace
	end
	
	private :drawChar
end
